// Copyright (C) 2009 - 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function [t,msg] = scibench_benchfun ( name , __benchfun_fun__ , iargs , nlhs , kmax , verbose )
  // Benchmarks a function and measure its performance.
  //
  // Calling Sequence
  //   [t,msg] = scibench_benchfun ( name , fun , iargs , nlhs , kmax , verbose )
  //
  // Parameters
  //   name : a 1 x 1 matrix of strings, the name of the function to be executed
  //   fun : a function, the function to be executed
  //   iargs : a list, the list of input arguments for the function
  //   nlhs : a 1 x 1 matrix of floating point integers, the number of output arguments of the function, must be greater than 0
  //   kmax : a 1 x 1 matrix of floating point integers, the number of executions, must be greater than 1
  //   verbose : a 1x1 matrix of booleans, set to %t to display the result
  //   t : a kmax x 1 matrix of doubles, the system times, in seconds, required to execute the function
  //   msg : a 1 x 1 matrix of strings, a message summarizing the benchmark
  //
  // Description
  //   This function is designed to be used when measuring the 
  //   performance of a function.
  //   It uses the timer function to measure the system time.
  //   The function is executed kmax times and the performance is 
  //   gathered into the matrix t.
  //   The message summarizes the test and contains the mean,
  //   min and max of the times.
  //
  // Examples
  // function c = pascalup_col (n)
  //   // Pascal up matrix.
  //   // Column by column version
  //   c = eye(n,n)
  //   c(1,:) = ones(1,n)
  //   for i = 2:(n-1)
  //     c(2:i,i+1) = c(1:(i-1),i)+c(2:i,i)
  //   end
  // endfunction
  // [t,msg] = scibench_benchfun ( "pascalup_col" , pascalup_col , list(100) , 1 , 10, %t )
  //
  // Authors
  //   2010 - DIGITEO - Michael Baudin
  

    [lhs, rhs] = argn()
    apifun_checkrhs ( "scibench_benchfun" , rhs , 6 )
    apifun_checklhs ( "scibench_benchfun" , lhs , 0:2 )
    //
    // Check Type
    apifun_checktype ( "scibench_benchfun" , name , "name" , 1 , "string" )
    apifun_checktype ( "scibench_benchfun" , __benchfun_fun__ , "benchfun_fun" , 2 , ["function" "list" "fptr"] )
    apifun_checktype ( "scibench_benchfun" , iargs , "iargs" , 3 , "list" )
    apifun_checktype ( "scibench_benchfun" , nlhs , "nlhs" , 4 , "constant" )
    apifun_checktype ( "scibench_benchfun" , kmax , "kmax" , 5 , "constant" )
    apifun_checktype ( "scibench_benchfun" , verbose , "verbose" , 6 , "boolean" )
    //
    // Check Size
    apifun_checkscalar ( "scibench_benchfun" , name , "name" , 1 )
    apifun_checkscalar ( "scibench_benchfun" , nlhs , "nlhs" , 4 )
    apifun_checkscalar ( "scibench_benchfun" , kmax , "kmax" , 5 )
    apifun_checkscalar ( "scibench_benchfun" , verbose , "verbose" , 6 )
    //
    // Check Content
    apifun_checkgreq ( "scibench_benchfun" , nlhs , "nlhs" , 4 , 0 )
    apifun_checkgreq ( "scibench_benchfun" , kmax , "kmax" , 5 , 1 )
    // TODO : check that these are floating point integers

  //
  // Compute the intstruction string to be launched
  ni = length ( iargs )
  instr = ""
  // Put the LHS arguments
  if ( nlhs > 0 ) then
    instr = instr + "["
  end
  for i = 1 : nlhs
    if ( i > 1 ) then
      instr = instr + ","
    end
    instr = instr + "x" + string(i)
  end
  if ( nlhs > 0 ) then
    instr = instr + "]"
  end
  if ( nlhs > 0 ) then
    instr = instr + "="
  end
  // Put the RHS arguments
  instr = instr + "__benchfun_fun__("
  for i = 1 : ni
    if ( i > 1 ) then
      instr = instr + ","
    end
    instr = instr + "iargs("+string(i)+")"
  end
  instr = instr + ")"
  //
  // Loop over the tests
  for k = 1 : kmax
    // Call the function
    timer()
    ierr = execstr ( instr , "errcatch" )
    t(k) = timer()
    if ( ierr <> 0 ) then
      errmsg = lasterror()
      error(sprintf(gettext("%s: Failed to run function: %s.\n"), ..
        "scibench_benchfun", errmsg))
    end
  end
  if ( verbose ) then
    msg = msprintf("%s: %d iterations, mean=%f, min=%f, max=%f\n",name,kmax,mean(t),min(t),max(t))
    mprintf("%s\n",msg)
  end
endfunction

